<?php
/*****************************************************************************
 *
 * ezcGraphPieChart_2.php - Gadget for NagVis to diplay pie charts
 *
 * Copyright (c) 2004-2009 NagVis Project (Contact: lars@vertical-visions.de, michael_luebben@web.de)
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************
 *
 * This is a simple gadget for NagVis. A gadget is a script which generates a
 * dynamic image for visualizing things as graphics in NagVis.
 *
 * The gadget gets its data from the NagVis frontend by parameters. This
 * gadget only needs the "perfdata" parameter. NagVis also passes the
 * following parameters to the gadgets:
 *  - name1:     Hostname
 *  - name2:     Service description
 *  - state:     Current state
 *  - stateType: Current state type (soft/hard)
 *
 *****************************************************************************/

// Load gadget core functions
require('./gadgets_core.php');

/*******************************************************************************
 * Start gadget main code
 ******************************************************************************/

header("Content-type: image/png");

//==========================================
// Set Minimum, Default, and Maximum values.
//==========================================

$min = 0;
$max = -1;
$default = 0; 
 
/* Now read the parameters */

$aOpts = Array('name1', 'name2', 'state', 'stateType', 'perfdata');
$aPerfdata = Array();

/**
 * Needed:
 *  perfdata=load1=0.960;5.000;10.000;0; load5=0.570;4.000;6.000;0; load15=0.540;3.000;4.000;0;
 *
 * Optional
 *  name1=localhost
 *  name2=Current Load
 *  state=OK
 *  stateType=HARD
 */

if(isset($_GET['perfdata']) && $_GET['perfdata'] != '') {
	$aOpts['perfdata'] = $_GET['perfdata'];
} else {
	errorBox('ERROR: The needed parameter "perfdata" is missing.');
}

if(isset($_GET['name1']) && $_GET['name1'] != '') {
	$aOpts['name1'] = $_GET['name1'];
}

if(isset($_GET['name2']) && $_GET['name2'] != '') {
	$aOpts['name2'] = $_GET['name2'];
}

if(isset($_GET['state']) && $_GET['state'] != '') {
	$aOpts['state'] = $_GET['state'];
}

if(isset($_GET['stateType']) && $_GET['stateType'] != '') {
	$aOpts['stateType'] = $_GET['stateType'];
}

/* Now parse the perfdata */
$aPerfdata = parsePerfdata($aOpts['perfdata']);

// This gadget is simple and dirty, it only recognizes the first dataset of
// performance data

$value = $aPerfdata[0]['value'];
$uom = $aPerfdata[0]['uom'];
$warn = $aPerfdata[0]['warning'];
$crit = $aPerfdata[0]['critical'];
$min = $aPerfdata[0]['min'];
$max = $aPerfdata[0]['max'];
$free = $max - $value;


// Draw pie chart and create image ouput

require_once "./ezcGraph/Base/src/base.php"; // dependent on installation method, see below
function __autoload( $className )
{
        ezcBase::autoload( $className );
}
$graph = new ezcGraphPieChart();
$graph->palette = new ezcGraphPaletteBlack();
$graph->palette->dataSetColor = array(
        '#007400',
        '#C70000',
);
$graph->title = 'Disk statistics for Host '.$aOpts['name1'].' / '.$aOpts['name2'];
$graph->options->label = '%2$.2f'.$uom.' (%3$.1f%%)';
$graph->data['Disk statistics for Host '.$aOpts['name1'].' / '.$aOpts['name2']] = new ezcGraphArrayDataSet( array(
    'Free' => $free,
    'Used' => $value,
) );
$graph->data['Disk statistics for Host '.$aOpts['name1'].' / '.$aOpts['name2']]->highlight['Used'] = true;
$graph->renderer = new ezcGraphRenderer3d();
$graph->renderer->options->moveOut = .2;
$graph->renderer->options->pieChartOffset = 63;
$graph->renderer->options->pieChartGleam = .3;
$graph->renderer->options->pieChartGleamColor = '#FFFFFF';
$graph->renderer->options->pieChartGleamBorder = 2;
$graph->renderer->options->pieChartShadowSize = 3;
$graph->renderer->options->pieChartShadowColor = '#000000';
$graph->renderer->options->legendSymbolGleam = .5;
$graph->renderer->options->legendSymbolGleamSize = .9;
$graph->renderer->options->legendSymbolGleamColor = '#FFFFFF';
$graph->renderer->options->pieChartSymbolColor = '#BABDB688';
$graph->driver = new ezcGraphGdDriver();
$graph->options->font = './ezcGraph/Fonts/tutorial_font.ttf';
$graph->renderToOutput( 400, 150);
?>
